/*
 * Copyright (C) 2011, 2012, 2013 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef WKGeolocationManager_h
#define WKGeolocationManager_h

#include <WebKit2/WKBase.h>

#ifdef __cplusplus
extern "C" {
#endif

// Provider.
typedef void (*WKGeolocationProviderStartUpdatingCallback)(WKGeolocationManagerRef geolocationManager, const void* clientInfo);
typedef void (*WKGeolocationProviderStopUpdatingCallback)(WKGeolocationManagerRef geolocationManager, const void* clientInfo);
typedef void (*WKGeolocationProviderSetEnableHighAccuracyCallback)(WKGeolocationManagerRef geolocationManager, bool enabled, const void* clientInfo);

typedef struct WKGeolocationProviderBase {
    int                                                                 version;
    const void *                                                        clientInfo;
} WKGeolocationProviderBase;

typedef struct WKGeolocationProviderV0 {
    WKGeolocationProviderBase                                           base;

    // Version 0.
    WKGeolocationProviderStartUpdatingCallback                          startUpdating;
    WKGeolocationProviderStopUpdatingCallback                           stopUpdating;
} WKGeolocationProviderV0;

typedef struct WKGeolocationProviderV1 {
    WKGeolocationProviderBase                                           base;

    // Version 0.
    WKGeolocationProviderStartUpdatingCallback                          startUpdating;
    WKGeolocationProviderStopUpdatingCallback                           stopUpdating;

    // Version 1.
    WKGeolocationProviderSetEnableHighAccuracyCallback                  setEnableHighAccuracy;
} WKGeolocationProviderV1;

enum { kWKGeolocationProviderCurrentVersion WK_ENUM_DEPRECATED("Use an explicit version number instead") = 1 };
typedef struct WKGeolocationProvider {
    int                                                                 version;
    const void *                                                        clientInfo;
    WKGeolocationProviderStartUpdatingCallback                          startUpdating;
    WKGeolocationProviderStopUpdatingCallback                           stopUpdating;

    // Version 1.
    WKGeolocationProviderSetEnableHighAccuracyCallback                  setEnableHighAccuracy;
} WKGeolocationProvider WK_DEPRECATED("Use an explicit versioned struct instead");


WK_EXPORT WKTypeID WKGeolocationManagerGetTypeID();

WK_EXPORT void WKGeolocationManagerSetProvider(WKGeolocationManagerRef geolocationManager, const WKGeolocationProviderBase* provider);

WK_EXPORT void WKGeolocationManagerProviderDidChangePosition(WKGeolocationManagerRef geolocationManager, WKGeolocationPositionRef position);
WK_EXPORT void WKGeolocationManagerProviderDidFailToDeterminePosition(WKGeolocationManagerRef geolocationManager);

WK_EXPORT void WKGeolocationManagerProviderDidFailToDeterminePositionWithErrorMessage(WKGeolocationManagerRef geolocationManager, WKStringRef errorMessage);

#ifdef __cplusplus
}
#endif

#endif /* WKGeolocationManager_h */
