/*
 * Copyright (C) 2006, 2007, 2010 Apple Inc. All rights reserved.
 * Copyright (C) 2006 Samuel Weinig <sam.weinig@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

interface HTMLTableElement : HTMLElement {
    [SetterRaisesException] attribute HTMLTableCaptionElement caption;
    [SetterRaisesException] attribute HTMLTableSectionElement tHead;
    [SetterRaisesException] attribute HTMLTableSectionElement tFoot;

    readonly attribute HTMLCollection rows;
    readonly attribute HTMLCollection tBodies;
    [Reflect] attribute DOMString align;
    [Reflect] attribute DOMString bgColor;
    [Reflect] attribute DOMString border;
    [Reflect] attribute DOMString cellPadding;
    [Reflect] attribute DOMString cellSpacing;

    [Reflect] attribute DOMString frame;

    [Reflect] attribute DOMString rules;
    [Reflect] attribute DOMString summary;
    [Reflect] attribute DOMString width;

    HTMLElement createTHead();
    void deleteTHead();
    HTMLElement createTFoot();
    void deleteTFoot();
    HTMLElement createTBody();
    HTMLElement createCaption();
    void deleteCaption();

    [RaisesException] HTMLElement insertRow([Default=Undefined] optional long index);
    [RaisesException] void deleteRow([Default=Undefined] optional long index);
};

